# Nick's web site: Ruby helpers for processing
#
# Copyright © 2018-2021 Nick Bowler
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

require 'nokogiri'

use_helper Nanoc::Helpers::Breadcrumbs
use_helper Nanoc::Helpers::Blogging

Xmlns = {
    'xhtml' => 'http://www.w3.org/1999/xhtml'
}.freeze
$counters = {}

def to_xhtml(subpath = "", item = @item)
    if item.identifier =~ '/index.*'
        ret =  "/" + subpath + "/index.xhtml"
    else
        ret = item.identifier.without_ext + "/" + subpath + "/index.xhtml"
    end

    return ret.gsub(/\/+/, "/")
end

def item_source(item = @item)
    filename = "content" + item.identifier
    filebase = filename.chomp(File.extname(filename))

    [ if item.binary? then filebase + ".yaml" end,
      filename].compact.find { |f| File.file? f }
end

def item_uri(item = @item, rep: :default)
    return item.path(rep: rep).sub(%r{/index[.][^.]*$}, "/")
end

def rep_uri(rep = @rep)
    return rep.path.sub(%r{/index[.][^.]*$}, "/")
end

def find_license(license)
    matches = @items.find_all("/license/" + license + ".*")

    raise("License not defined: " + license) if !matches.length
    return matches.sort_by { |item| item.identifier } [0]
end

# Return the first paragraph of the given item as a string.
def item_longdesc(item)
    xml = Nokogiri::XML("<body xmlns='" + Xmlns["xhtml"] + "'>" +
                        item.compiled_content(snapshot: :rawbody) +
                        "</body>")

    p = xml.xpath('//xhtml:p', Xmlns)
    if p.empty? then nil else p[0].xpath('string(.)') end
end

def counter(name = :default, item = @item)
    $counters[item] ||= {}
    $counters[item][name] ||= 0

    name.to_s.capitalize + " " + ($counters[item][name] += 1).to_s
end

def human_filesize(size)
    units = ["B", "KiB", "MiB", "GiB"]
    prec = 0

    for unit in units
        if (size < 1024)
            break
        end

        size /= 1024.0
        prec = 1
    end

    sprintf("%.*f %s", prec, size + 0.05, unit)
end
